{{

______________________________________________________________________________
********************************************************* File starts here ***
Zerocat Chipflasher --- Flash free firmware, kick the Management Engine.

Copyright (C) 2020, 2021, 2022  Kai Mertens <kmx@posteo.net>

File kick2.spin --- The top object file for this firmware2 project.

This file is part of Zerocat Chipflasher.

See end of file for terms of use.

******************************************************************************
}}


{{


# Object Configuration (set via Makefile)

Tell us which interface to use:

* connect
* flashrom
* terminal
* none
}}
'
'
'   Maintain the configuration token:
'
'   * enable : "^# \t "
'   * disable: "^'~ #"
'
'
# 	   define _CFG_CONNECT
'~ #   define _CFG_FLASHROM
'~ #   define _CFG_TERMINAL
'
'
#   ifdef _CFG_CONNECT
#       warning Interface: connect
#   endif
#   ifdef _CFG_FLASHROM
#       warning Interface: flashrom
#   endif
#   ifdef _CFG_TERMINAL
#       warning Interface: terminal
#   endif
#   ifndef _CFG_CONNECT
#       ifndef _CFG_FLASHROM
#           ifndef _CFG_TERMINAL
#               warning Interface: nointerface
#           endif
#       endif
#   endif


{{


# Object Summary
}}
OBJ     'This directive triggers the compiler to insert a summary.
{{


# Objects

}}
        'objects without VAR block
        pins            : "pins"                ''Load pin configuration object.
        asc             : "ASCII"               ''Load ASCII object.
        error           : "error"               ''Load error object.

        'objects with VAR block
#   ifdef _CFG_CONNECT
        menu            : "menu"                ''Load menu object.
#   endif
#   ifdef _CFG_TERMINAL
        menu            : "menu"                ''Load menu object.
#   endif
#   ifdef _CFG_FLASHROM
        serprog         : "serprog"             ''Load serprog object.
#   endif


CON
{{


# Constants

}}
        ''System Speed
        _XINFREQ            = 5_000_000                     ''  _XINFREQ  --- External Crystal Frequency of 5MHz
        _CLKMODE            = XTAL1 + PLL16X                ''  _CLKMODE  --- Low Speed Crystal x 16    = 80MHz
        '_CLKFREQ is calculated automatically from above settings


        ''Basic Timing
        CYCL_MIN            = 381


        ''Project's Memory Usage
        '
        'Block runtime memory to have hands free later on.
        '_STACK and _FREE must sum up to under MAX_TOTAL longs.
        'Set RUNTIME_STACK to zero in order to create an informative compile error.
        'Set RUNTIME_RAM to zero in order to create an informative compiler error.
        '
        MAX_TOTAL           = 8192                                      ''  MAX_TOTAL
        MAX_STACK           = 4096                                      ''  MAX_STACK
        RUNTIME_RAM         = 2950                                      ''  RUNTIME_RAM
        RUNTIME_STACK       = 2900                                      ''  RUNTIME_STACK
        _STACK              = MAX_STACK - RUNTIME_STACK                 ''  _STACK
        _FREE               = (MAX_TOTAL - MAX_STACK) - RUNTIME_RAM     ''  _FREE


{{


# Functions
}}


PUB     kick2 | errcode, RST_inhibit, boardcfg, vboard

        ''This is the Chipflasher's new firmware, coded in Spin/PASM.

        'get board configuration, first statement!
        boardcfg := pins.get_board_config
        vboard := pins.get_board_version(boardcfg)

        'get pin for RST inhibit functionality
        RST_inhibit := pins.get_pin_RSTinhibit(boardcfg)

        'init
        DIRA[RST_inhibit]~~
        DIRA[pins#PIN_D1..pins#PIN_D3]~~

        'hello world!
        REPEAT vboard
            error.display(error#ERR_HELLO)

        'disable RST until we are done.
        OUTA[RST_inhibit]~~                             'disable pin RST
        errcode := error#ERR_RST

        'start counting of reset pulses
        IF vboard > 1
            FRQA := 1
            PHSA := 0
            CTRA := %0_01010_000 << 23 + 0 << 9 + pins#RS232_RST << 0

        'start interface
        error.display(errcode)                          'display status

#   ifdef _CFG_FLASHROM
        IF pins.check_cable == TRUE
            'start communication: try flashrom utility
            errcode := serprog.start(boardcfg)
        ELSE
            errcode := error#ERR_CABLE
#   endif
#   ifdef _CFG_CONNECT
        IF pins.check_cable == TRUE
            'start communication: try connect utility
            errcode := menu.handshake_connect(20000)        'timeout: 20s
            IF errcode == error#ERR_SUCCESS
                errcode := menu.start(menu#MODE_CONNECT)
        ELSE
            errcode := error#ERR_CABLE
#   endif
#   ifdef _CFG_TERMINAL
        IF pins.check_cable == TRUE
            'start communication: assume propeller terminal
            errcode := menu.start(menu#MODE_TERMINAL)
        ELSE
            errcode := error#ERR_CABLE
#   endif
#   ifndef _CFG_FLASHROM
#       ifndef _CFG_CONNECT
#           ifndef _CFG_TERMINAL
        errcode := error#ERR_INTERFACE
#           endif
#       endif
#   endif

        'end of program:
        CASE vboard
            1:
                REPEAT 1
                    error.display(errcode)            'display error code
                OUTA[RST_inhibit]~
            OTHER:
                '1. Display error status (at least once) continuously
                '2. Release RST_inhibit
                REPEAT
                    error.display(errcode)            'display error code
                    IF OUTA[RST_inhibit]
                        IF PHSA > 1
                            OUTA[RST_inhibit]~


{{


******************************************************************************
Terms of Use:

Zerocat Chipflasher is free software: you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

Zerocat Chipflasher is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along
with Zerocat Chipflasher.  If not, see <http://www.gnu.org/licenses/>.

************************************************************** End of File ***
}}
